#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAIN_C
#include "exitfail.h"

#define BASENAME(p)   ((strrchr((p), '/') ? : ((p) - 1)) + 1)

/**
 * main関数
 * @param argc 引数なしの場合はusage表示のみ
 * @param argv 第1引数として接続待ちポートを指定
 * @return exit値
 */
int main(int argc, char *argv[])
{
	in_port_t listen_port;
	static int listen_fd, accept_fd; /* ソケットファイルディスクリプタ */
	struct sockaddr_in server_addr, client_addr;
	socklen_t addr_len;
	char message[] = "Hello!\r\n";
	ssize_t ret;

	exitfail_init();

	/* 引数が指定されなかった場合、usage表示して終了 */
	if (argc < 2) {
		printf("Usage: %s <port>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	ret = strtoul(argv[1], NULL, 10);
	if (ret == LONG_MIN || ret == LONG_MAX)
		exitfail_errno("strtol");
	if (ret < 49152 || 65535 < ret)
		exitfail("Specify the port 49152-65535\n");
	listen_port = ret;

	/* 接続待ち用のソケットを作成 */
	listen_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (listen_fd < 0)
		exitfail_errno("open");

	/* アドレスとポートを割り当て */
	memset(&server_addr, 0, sizeof(server_addr));
	server_addr.sin_family = AF_INET; /* IPv4インターネットプロトコル */
	server_addr.sin_addr.s_addr = INADDR_ANY; /* 任意のアドレス */
	server_addr.sin_port = htons(listen_port); /* 接続を待つポート */
	addr_len = sizeof(server_addr);
	ret = bind(listen_fd, (struct sockaddr *)&server_addr, addr_len);
	if (ret < 0)
		exitfail_errno("bind");

	/* クライアントからの接続を待つ */
	ret = listen(listen_fd, SOMAXCONN);
	if (ret < 0)
		exitfail_errno("listen");

	/* クライアントからの接続を受け付けて、入出力用のソケットを作成 */
	accept_fd = accept(listen_fd, (struct sockaddr *)&client_addr,
			   &addr_len);
	if (accept_fd < 0)
		exitfail_errno("accept");

	/* これ以上の接続を受け付けないため、接続待ち用ソケットを破棄 */
	close(listen_fd);

	/* メッセージを送信 */
	ret = write(accept_fd, message, strlen(message));
	if (ret < 0)
		exitfail_errno("write");

	/* 入出力用のソケットを破棄 */
	close(accept_fd);

	return EXIT_SUCCESS;
}
